

#ifndef HISTORY_H
#define HISTORY_H

#include "infodbclass.h"
#include "text_t.h"
#include "recptproto.h"
//#include <vector.h>
#include <map.h>

struct HistRecord {
  text_t entry;
  bool save;
};

struct ltinttext_t
{
  bool operator()(const text_t &t1, const text_t &t2) const
  { return t1.getint() < t2.getint(); }
};


typedef map<text_t, HistRecord, ltinttext_t> HistRecordmap;

// returns true on success (in which case historyinfo will contain
// the information for this history)
bool get_history_info ( const text_t &userid, text_tarray &historyinfo);

// returns true on success
bool set_history_info (const text_t &userid, const text_t &historyinfo);

bool delete_all_history_info (const text_t &userid);

// deletes only the entries specified
bool delete_history_info(const text_t &userid, const text_t &deletemode,
			 const text_t &selection);

// takes a text_t with x-y, and retrieves the x and y params
bool get_selection_params (text_t &data, int &start, int &stop);

bool expand_query(HistRecord &query, HistRecordmap &records);

// combine query looks for #x and replaces it with the query
// string from the xth entry. THe first one looks up the xth
// query in the database (used before running a query), the 
// second version looks it up inthe records map (used in
// delete history -are doing a lotof #x replacing at once.
bool combine_query(text_t &userid, text_t &query);
bool combine_query(text_t &userid, HistRecordmap  &records);

// retrieves the value of an arg (key) from a list of cgi args
// in the form a=q&b=1&...
void parse_saved_args(text_t &args, text_t &key, text_t &value);

// extracts out the components of an entry in the database
// currently:
// querynum:numdocs cgiargs
void split_saved_query(text_t &query, text_t &querynum, text_t &numdocs, text_t &cgiargs);

// just extracts the query number out of the entry
void get_query_num(text_t &query, text_t &querynum);

// takes the cgiarg string, and outputs it in some form useful
// to the user
void format_user_info (text_t &cgiargs, text_t &info, 
		       recptprotolistclass *protos, ostream &logout);

#endif


